// *****************************************************************
// ****                      Synth CORE B                       ****
// ****                  www.tolaemon.com/scoreb                ****
// ****                       Source code                       ****
// ****                       13/11/2011                        ****
// ****                     Jordi Bartolome                     ****
// ****                                                         ****
// **** IMPORTANT:                                              ****
// **** Using parts of this code means accepting all conditions ****
// **** exposed on the web: www.tolaemon.com/scoreb             ****
// ****                                                         ****
// **** Parts of this code have been taken from the demo source ****
// **** code of EVK1100 Atmel's evaluation board				****
// *****************************************************************


#include <avr32/io.h>
#include "board.h"
#include "gpio.h"
#include "usart.h"

// scheduler includes.
#include "FreeRTOS.h"
#include "queue.h"
#include "task.h"

// synth core B includes
#include "defines.h"
#include "com.h"

// Constants to setup and access the USART.
#define serINVALID_COMPORT_HANDLER        ( ( xComPortHandle ) 0 )
#define serINVALID_QUEUE                  ( ( xQueueHandle ) 0 )
#define serHANDLE                         ( ( xComPortHandle ) 1 )
#define serNO_BLOCK                       ( ( portTickType ) 0 )

// Queues used to hold received characters, and characters waiting to be transmitted.
static xQueueHandle x_serial_232_rx;
static xQueueHandle x_serial_232_tx;

static xQueueHandle x_serial_USB_rx;
static xQueueHandle x_serial_USB_tx;

static xQueueHandle x_serial_MIDI_rx;
static xQueueHandle x_serial_MIDI_tx;

// COMs configuration vars
struct {
	portCHAR com_serial_232_initialized;
	portCHAR com_serial_USB_initialized;
	portCHAR com_serial_MIDI_initialized;
} com_config;


#if __GNUC__
	__attribute__((__noinline__))
#elif __ICCAVR32__
	#pragma optimize = no_inline
#endif
static portBASE_TYPE prvUSART_ISR_SERIAL_232_NonNakedBehaviour( void )
{

	// Now we can declare the local variables.
	signed portCHAR     cChar;
	portBASE_TYPE     xTaskWokenByTx = pdFALSE, xTaskWokenByRx = pdFALSE;
	unsigned portLONG     ulStatus;

	volatile avr32_usart_t  *usart = serial_232_PORT_USART;

	portBASE_TYPE retstatus;

	// What caused the interrupt?
	ulStatus = usart->csr & usart->imr;

	if (ulStatus & AVR32_USART_CSR_TXRDY_MASK)
	{
		// The interrupt was caused by the THR becoming empty.  Are there any more characters to transmit?
		// Because FreeRTOS is not supposed to run with nested interrupts, put all OS calls in a critical section
		portENTER_CRITICAL();
			retstatus = xQueueReceiveFromISR(x_serial_232_tx, &cChar, &xTaskWokenByTx );
		portEXIT_CRITICAL();

		if (retstatus == pdTRUE){
			// A character was retrieved from the queue so can be sent to the THR now.
			usart->thr = cChar;
		}else{
			// Queue empty, nothing to send so turn off the Tx interrupt.
			usart->idr = AVR32_USART_IDR_TXRDY_MASK;
		}
	}

	if (ulStatus & AVR32_USART_CSR_RXRDY_MASK)
	{
		// The interrupt was caused by the receiver getting data.
		cChar = usart->rhr; //TODO

		// Because FreeRTOS is not supposed to run with nested interrupts, put all OS calls in a critical section .
		portENTER_CRITICAL();
			retstatus = xQueueSendFromISR(x_serial_232_rx, &cChar, pdFALSE);
		portEXIT_CRITICAL();

		if( retstatus ){
			xTaskWokenByRx = pdTRUE;
		}
	}

	// The return value will be used by portEXIT_SWITCHING_ISR() to know if it should perform a vTaskSwitchContext().
	return ( xTaskWokenByTx || xTaskWokenByRx );

}//prvUSART_ISR_SERIAL_232_NonNakedBehaviour



#if __GNUC__
	__attribute__((__noinline__))
#elif __ICCAVR32__
	#pragma optimize = no_inline
#endif
static portBASE_TYPE prvUSART_ISR_SERIAL_USB_NonNakedBehaviour( void )
{
	// Now we can declare the local variables.
	signed portCHAR     cChar;
	portBASE_TYPE     xTaskWokenByTx = pdFALSE, xTaskWokenByRx = pdFALSE;
	unsigned portLONG     ulStatus;

	volatile avr32_usart_t  *usart = serial_usb_PORT_USART;

	portBASE_TYPE retstatus;

	// What caused the interrupt?
	ulStatus = usart->csr & usart->imr;

	if (ulStatus & AVR32_USART_CSR_TXRDY_MASK)
	{
		// The interrupt was caused by the THR becoming empty.  Are there any more characters to transmit?
		// Because FreeRTOS is not supposed to run with nested interrupts, put all OS calls in a critical section .
		portENTER_CRITICAL();
			retstatus = xQueueReceiveFromISR( x_serial_USB_tx, &cChar, &xTaskWokenByTx );
		portEXIT_CRITICAL();

		if (retstatus == pdTRUE){
			// A character was retrieved from the queue so can be sent to the THR now.
			usart->thr = cChar;
		}else{
			// Queue empty, nothing to send so turn off the Tx interrupt.
			usart->idr = AVR32_USART_IDR_TXRDY_MASK;
		}
	}

	if (ulStatus & AVR32_USART_CSR_RXRDY_MASK){
		// The interrupt was caused by the receiver getting data.
		cChar = usart->rhr; //TODO

		// Because FreeRTOS is not supposed to run with nested interrupts, put all OS calls in a critical section .
		portENTER_CRITICAL();
			retstatus = xQueueSendFromISR(x_serial_USB_rx, &cChar, pdFALSE);
		portEXIT_CRITICAL();

		if( retstatus ){
			xTaskWokenByRx = pdTRUE;
		}
	}

	// The return value will be used by portEXIT_SWITCHING_ISR() to know if it should perform a vTaskSwitchContext().
	return ( xTaskWokenByTx || xTaskWokenByRx );

}//prvUSART_ISR_SERIAL_USB_NonNakedBehaviour



#if __GNUC__
	__attribute__((__noinline__))
#elif __ICCAVR32__
	#pragma optimize = no_inline
#endif
static portBASE_TYPE prvUSART_ISR_SERIAL_MIDI_NonNakedBehaviour( void )
{

	// Now we can declare the local variables.
	signed portCHAR     cChar;
	portBASE_TYPE     xTaskWokenByTx = pdFALSE, xTaskWokenByRx = pdFALSE;
	unsigned portLONG     ulStatus;

	volatile avr32_usart_t  *usart = serial_midi_PORT_USART;

	portBASE_TYPE retstatus;

	// What caused the interrupt?
	ulStatus = usart->csr & usart->imr;

	if (ulStatus & AVR32_USART_CSR_TXRDY_MASK)
	{
		// The interrupt was caused by the THR becoming empty.  Are there any more characters to transmit?
		// Because FreeRTOS is not supposed to run with nested interrupts, put all OS calls in a critical section .
		portENTER_CRITICAL();
			retstatus = xQueueReceiveFromISR( x_serial_MIDI_tx, &cChar, &xTaskWokenByTx );
		portEXIT_CRITICAL();

		if (retstatus == pdTRUE){
			// A character was retrieved from the queue so can be sent to the THR now.
			usart->thr = cChar;
		}else{
			// Queue empty, nothing to send so turn off the Tx interrupt.
			usart->idr = AVR32_USART_IDR_TXRDY_MASK;
		}
	}

	if (ulStatus & AVR32_USART_CSR_RXRDY_MASK){
		// The interrupt was caused by the receiver getting data.
		cChar = usart->rhr; //TODO

		// Because FreeRTOS is not supposed to run with nested interrupts, put all OS calls in a critical section .
		portENTER_CRITICAL();
			retstatus = xQueueSendFromISR( x_serial_MIDI_rx, &cChar, pdFALSE);
		portEXIT_CRITICAL();

		if( retstatus ){
			xTaskWokenByRx = pdTRUE;
		}
	}

	// The return value will be used by portEXIT_SWITCHING_ISR() to know if it should perform a vTaskSwitchContext().
	return ( xTaskWokenByTx || xTaskWokenByRx );

}//prvUSART_ISR_SERIAL_MIDI_NonNakedBehaviour



//
// USART interrupt service routine.
//
#if __GNUC__
	__attribute__((__naked__))
#elif __ICCAVR32__
	#pragma shadow_registers = full   // Naked.
#endif
static void vUSART_ISR_SERIAL_232( void )
{

	// This ISR can cause a context switch, so the first statement must be a
	//call to the portENTER_SWITCHING_ISR() macro.  This must be BEFORE any
	//variable declarations.
	portENTER_SWITCHING_ISR();

	prvUSART_ISR_SERIAL_232_NonNakedBehaviour();

	// Exit the ISR.  If a task was woken by either a character being received
	//or transmitted then a context switch will occur.
	portEXIT_SWITCHING_ISR();

}



#if __GNUC__
	__attribute__((__naked__))
#elif __ICCAVR32__
	#pragma shadow_registers = full   // Naked.
#endif
static void vUSART_ISR_SERIAL_USB( void )
{
	/* This ISR can cause a context switch, so the first statement must be a
	call to the portENTER_SWITCHING_ISR() macro.  This must be BEFORE any
	variable declarations. */
	portENTER_SWITCHING_ISR();

	prvUSART_ISR_SERIAL_USB_NonNakedBehaviour();

	/* Exit the ISR.  If a task was woken by either a character being received
	or transmitted then a context switch will occur. */
	portEXIT_SWITCHING_ISR();
}



#if __GNUC__
	__attribute__((__naked__))
#elif __ICCAVR32__
	#pragma shadow_registers = full   // Naked.
#endif
static void vUSART_ISR_SERIAL_MIDI( void )
{
	// This ISR can cause a context switch, so the first statement must be a
	// call to the portENTER_SWITCHING_ISR() macro.  This must be BEFORE any
	// variable declarations.
	portENTER_SWITCHING_ISR();

	prvUSART_ISR_SERIAL_MIDI_NonNakedBehaviour();

	// Exit the ISR.  If a task was woken by either a character being received
	// or transmitted then a context switch will occur.
	portEXIT_SWITCHING_ISR();
}



// Synth Core  12-10-2011  Tolaemon
// Function: void COM_reset()
// Function wich resets COM control vars. It is used when intializing
// the system to grant all vars are on "initial conditions"
// Receives:
// Returns:
void COM_reset(){

	com_config.com_serial_232_initialized=FALSE;
	com_config.com_serial_USB_initialized=FALSE;
	com_config.com_serial_MIDI_initialized=FALSE;

}// COM_reset


// Synth Core  23-11-2011  Tolaemon
// Function: void COM_reset()
// Function which returns the state of the specified COM port
// Receives:
//   portCHAR com_number : with the number of the port that we want to
//   chec if is initialized or not
// Returns:
// 	 TRUE:  if the specified port is initialized
//   FALSE: if the specified port is NOT initialized
portCHAR COM_is_initalized(portCHAR com_number){

	switch (com_number){
		case COM_SERIAL_232:
			return com_config.com_serial_232_initialized;
			break;
		case COM_SERIAL_USB:
			return com_config.com_serial_USB_initialized;
			break;
		case COM_SERIAL_MIDI:
			return com_config.com_serial_MIDI_initialized;
			break;
		default:
			return FALSE;
			break;
	}//switch

}// COM_is_initalized


// Synth Core  12-10-2011  Tolaemon
// Function: signed portCHAR COM_init(portCHAR com_number){
// Function which initializes the specified COMuncations port
// Receives:
//    portUCHAR com_number: with the COMunication port we want to initialize
// Returns:
// 	 1: if the port has been properly initialized
//  -1:if the port could not be initialized
//  -2:if the specified port does not exist
signed portCHAR COM_init(portCHAR com_number,unsigned portLONG baudrate,unsigned portCHAR charlen, unsigned  portCHAR paritytype, unsigned portCHAR stopbits, unsigned portCHAR channelmode){

	signed portCHAR return_value=0;

	volatile avr32_usart_t  *usart;

	usart_options_t USART_OPTIONS;

	// USART connected to the MAX 232 ( USART 0 )
	static const gpio_map_t USART_232_GPIO_MAP =
	{
			{ serial_232_PORT_USART_RX_PIN, serial_232_PORT_USART_RX_FUNCTION },
			{ serial_232_PORT_USART_TX_PIN, serial_232_PORT_USART_TX_FUNCTION }
	};

	// USART connected to the FTDI USB ( USART 1 )
	static const gpio_map_t USART_USB_GPIO_MAP =
	{
			{ serial_usb_PORT_USART_RX_PIN, serial_usb_PORT_USART_RX_FUNCTION },
			{ serial_usb_PORT_USART_TX_PIN, serial_usb_PORT_USART_TX_FUNCTION }
	};

	// USART connected to the MIDI ( USART 2 )
	static const gpio_map_t USART_MIDI_GPIO_MAP =
	{
			{ serial_midi_PORT_USART_RX_PIN, serial_midi_PORT_USART_RX_FUNCTION },
			{ serial_midi_PORT_USART_TX_PIN, serial_midi_PORT_USART_TX_FUNCTION }
	};

	USART_OPTIONS.baudrate=baudrate;
	USART_OPTIONS.charlength=charlen;
	USART_OPTIONS.paritytype=paritytype;
	USART_OPTIONS.channelmode=channelmode;
	USART_OPTIONS.stopbits=stopbits;

	switch (com_number){

		case COM_SERIAL_232:

			if (com_config.com_serial_232_initialized==TRUE){

				// the port was already initialized
				return_value=-1;

			}else{

				usart = serial_232_PORT_USART;

				//
				// Configure USART.
				//

				// Create the queues used to hold Rx and Tx characters in 232 port
				x_serial_232_rx = xQueueCreate( COM_SERIAL_232_QUEUE_LENGTH, ( unsigned portBASE_TYPE ) sizeof( signed portCHAR ) );
				x_serial_232_tx = xQueueCreate( COM_SERIAL_232_QUEUE_LENGTH + 1, ( unsigned portBASE_TYPE ) sizeof( signed portCHAR ) );

				// Enable USART RXD & TXD pins.
				gpio_enable_module( USART_232_GPIO_MAP, sizeof( USART_232_GPIO_MAP ) / sizeof( USART_232_GPIO_MAP[0] ) );

				// Initialize USART in RS232 mode.
				usart_init_rs232(usart, &USART_OPTIONS, configPBA_CLOCK_HZ);

				// We're not fully done yet: disable receiver and transmitter.
				usart->cr |= AVR32_USART_CR_RXDIS_MASK | AVR32_USART_CR_TXDIS_MASK;

				// Register the USART interrupt handler to the interrupt controller and
				// enable the USART interrupt.
				INTC_register_interrupt((__int_handler)&vUSART_ISR_SERIAL_232, serial_232_PORT_USART_IRQ, COM_SERIAL_232_INTERRUPT_LEVEL);

				// Enable USART interrupt sources (but not Tx for now)...
				usart->ier = AVR32_USART_IER_RXRDY_MASK;

				// Enable receiver and transmitter...
				usart->cr |= AVR32_USART_CR_TXEN_MASK | AVR32_USART_CR_RXEN_MASK;

				com_config.com_serial_232_initialized=TRUE;

				return_value=1;

			}//if
			break;

		case COM_SERIAL_USB:

			if (com_config.com_serial_USB_initialized==TRUE){

				// the port was already initialized
				return_value=-1;

			}else{

				usart = serial_usb_PORT_USART;

				//
				// Configure USART.
				//

				// Create the queues used to hold Rx and Tx characters in serial USB port
				x_serial_USB_rx = xQueueCreate( COM_SERIAL_USB_QUEUE_LENGTH, ( unsigned portBASE_TYPE ) sizeof( signed portCHAR ) );
				x_serial_USB_tx = xQueueCreate( COM_SERIAL_USB_QUEUE_LENGTH + 1, ( unsigned portBASE_TYPE ) sizeof( signed portCHAR ) );

				// Enable USART RXD & TXD pins.
				gpio_enable_module( USART_USB_GPIO_MAP, sizeof( USART_USB_GPIO_MAP ) / sizeof( USART_USB_GPIO_MAP[0] ) );

				// Initialize USART in RS232 mode.
				usart_init_rs232(usart, &USART_OPTIONS, configPBA_CLOCK_HZ);

				// We're not fully done yet: disable receiver and transmitter.
				usart->cr |= AVR32_USART_CR_RXDIS_MASK | AVR32_USART_CR_TXDIS_MASK;

				// Register the USART interrupt handler to the interrupt controller and
				// enable the USART interrupt.
				INTC_register_interrupt((__int_handler)&vUSART_ISR_SERIAL_USB, serial_usb_PORT_USART_IRQ, COM_SERIAL_USB_INTERRUPT_LEVEL);

				 // Enable USART interrupt sources (but not Tx for now)...
				usart->ier = AVR32_USART_IER_RXRDY_MASK;

				// Enable receiver and transmitter...
				usart->cr |= AVR32_USART_CR_TXEN_MASK | AVR32_USART_CR_RXEN_MASK;

				com_config.com_serial_USB_initialized=TRUE;

				return_value=1;

			}//if
			break;

		case COM_SERIAL_MIDI:

			if (com_config.com_serial_MIDI_initialized==TRUE){

				// the port was already initialized
				return_value=-1;

			}else{

				usart = serial_midi_PORT_USART;

				//
				// Configure USART.
				//

				// Create the queues used to hold Rx and Tx characters in serial MIDI port
				x_serial_MIDI_rx = xQueueCreate( COM_SERIAL_MIDI_QUEUE_LENGTH, ( unsigned portBASE_TYPE ) sizeof( signed portCHAR ) );
				x_serial_MIDI_tx = xQueueCreate( COM_SERIAL_MIDI_QUEUE_LENGTH + 1, ( unsigned portBASE_TYPE ) sizeof( signed portCHAR ) );

				// Enable USART RXD & TXD pins.
				gpio_enable_module( USART_MIDI_GPIO_MAP, sizeof( USART_MIDI_GPIO_MAP ) / sizeof( USART_MIDI_GPIO_MAP[0] ) );

				// Initialize USART in RS232 mode.
				usart_init_rs232(usart, &USART_OPTIONS, configPBA_CLOCK_HZ);

				// We're not fully done yet: disable receiver and transmitter
				usart->cr |= AVR32_USART_CR_RXDIS_MASK | AVR32_USART_CR_TXDIS_MASK;

				// Register the USART interrupt handler to the interrupt controller and enable the USART interrupt.
				INTC_register_interrupt((__int_handler)&vUSART_ISR_SERIAL_MIDI, serial_midi_PORT_USART_IRQ, COM_SERIAL_MIDI_INTERRUPT_LEVEL);

				// Enable USART interrupt sources (but not Tx for now)...
				usart->ier = AVR32_USART_IER_RXRDY_MASK;

				// Enable receiver and transmitter...
				usart->cr |= AVR32_USART_CR_TXEN_MASK | AVR32_USART_CR_RXEN_MASK;

				com_config.com_serial_MIDI_initialized=TRUE;

				return_value=1;

			}//if
			break;

		default:
			return_value=-1;
			break;

	}//switch

	return return_value;

}//COM_init



// Synth Core  12-10-2011  Tolaemon
// Function: xSerialGetChar( UCHAR com_number, signed portCHAR *pcRxedChar, portTickType xBlockTime )
// Function which reads the chars that have been received through the specified COM port number
// Receives:
//    portCHAR com_number: with the COMunication number port we want to read chars
 //	  portTickType xBlockTime : with the time the routine will stay blocked waitng for a char
// Returns:
// By reference:
//   pcRxedChar: the received char ( if a char has been read )
// By value
// 	 -1: if no char has been received
//   1: if some char has been received
signed portCHAR COM_xSerialGetChar( portCHAR com_number, signed portCHAR *pcRxedChar, portTickType xBlockTime )
{

	switch (com_number){

		case COM_SERIAL_232:

			// Get the next character from the buffer.  Return false if no characters are available, or arrive before xBlockTime expires.
			if( xQueueReceive( x_serial_232_rx, pcRxedChar, xBlockTime ) ){
				return 1;
			}else{
				return -1;
			}
			break;

		case COM_SERIAL_USB:

			// Get the next character from the buffer.  Return false if no characters are available, or arrive before xBlockTime expires.
			if( xQueueReceive( x_serial_USB_rx, pcRxedChar, xBlockTime ) ){
				return 1;
			}else{
				return -1;
			}
			break;

		case COM_SERIAL_MIDI:

			// Get the next character from the buffer.  Return false if no characters are available, or arrive before xBlockTime expires.
			if( xQueueReceive( x_serial_MIDI_rx, pcRxedChar, xBlockTime ) ){
				return 1;
			}else{
				return -1;
			}
			break;

		default:
			return -1;
			break;

	}//switch


}//COM_xSerialGetChar



// Synth Core  12-10-2011  Tolaemon
// Function: signed portBASE_TYPE xSerialPutChar( UCHAR com_number, signed portCHAR cOutChar, portTickType xBlockTime )
// Function which writes a char into the specified COM port number
// Receives:
//    portCHAR com_number: with the COMunication number port we want to write a char in.
//	  portTickType xBlockTime : with the time the routine will stay blocked ?
//    signed portCHAR cOutChar: with the char we want to write into the COM
// Returns:
// By reference:
//   pcRxedChar: the received char ( if a char has been read )
// By value
// 	 -1: if the char could not be written
//   1: if the car could be written
signed portCHAR COM_xSerialPutChar( portCHAR com_number, signed portCHAR cOutChar, portTickType xBlockTime )
{
	volatile avr32_usart_t  *usart;

	// before sending, the state of the specified COM port is checked
	if (COM_is_initalized(com_number)==FALSE){

		return pdFAIL;

	}else{

		switch (com_number){

			case COM_SERIAL_232:

				usart = serial_232_PORT_USART;

				// Place the character in the queue of characters to be transmitted.
				if( xQueueSend( x_serial_232_tx, &cOutChar, xBlockTime ) != pdPASS ){
					return pdFAIL;
				}

				// Turn on the Tx interrupt so the ISR will remove the character from the
				//queue and send it.   This does not need to be in a critical section as
				// if the interrupt has already removed the character the next interrupt
				//will simply turn off the Tx interrupt again.
				usart->ier = (1 << AVR32_USART_IER_TXRDY_OFFSET);

				break;

			case COM_SERIAL_USB:

				usart = serial_usb_PORT_USART;

				// Place the character in the queue of characters to be transmitted.
				if( xQueueSend( x_serial_USB_tx, &cOutChar, xBlockTime ) != pdPASS ){
					return pdFAIL;
				}

				// Turn on the Tx interrupt so the ISR will remove the character from the
				//queue and send it.   This does not need to be in a critical section as
				// if the interrupt has already removed the character the next interrupt
				//will simply turn off the Tx interrupt again.
				usart->ier = (1 << AVR32_USART_IER_TXRDY_OFFSET);

				break;

			case COM_SERIAL_MIDI:

				usart = serial_midi_PORT_USART;

				// Place the character in the queue of characters to be transmitted.
				if( xQueueSend( x_serial_MIDI_tx, &cOutChar, xBlockTime ) != pdPASS ){
					return pdFAIL;
				}

				// Turn on the Tx interrupt so the ISR will remove the character from the
				//queue and send it.   This does not need to be in a critical section as
				// if the interrupt has already removed the character the next interrupt
				//will simply turn off the Tx interrupt again.
				usart->ier = (1 << AVR32_USART_IER_TXRDY_OFFSET);

				break;

			default:
				break;

		}//switch

		return pdPASS;

	}//if

}//xSerialPutChar



// Synth Core  12-10-2011  Tolaemon
// Function: void vSerialPutString( UCHAR com_number, const signed portCHAR * const pcString, unsigned portSHORT usStringLength )
// Function which writes a string into the COM specified with COM port number
// Receives:
//    portCHAR com_number: with the COMunication number port we want to write a char in.
//	  const signed portCHAR * const pcString: a pointer to the char array with the string we want to pring
//    unsigned portSHORT usStringLength: with the length of the string we want to print
// Returns:
// By reference:
// By value
void COM_vSerialPutString( portCHAR com_number, const signed portCHAR * const pcString, unsigned portSHORT usStringLength )
{
	signed portCHAR *pxNext;
    unsigned portSHORT us_counter;

	// NOTE: This implementation does not handle the queue being full as no block time is used!

	// before sending, the state of the specified COM port is checked
	if (COM_is_initalized(com_number)==TRUE){

		// Send each character in the string, one at a time.
		pxNext = ( signed portCHAR * ) pcString;
		us_counter=0;
		while ((us_counter<usStringLength)&&( *pxNext )){
			COM_xSerialPutChar( com_number, *pxNext, serNO_BLOCK );
			pxNext++;
			us_counter++;
		}//while

	}//if

}//vSerialPutString



// Synth Core  23-10-2011  Tolaemon
// Function: void COM_vSerialClose( portCHAR com_number )
// Function which closes the specifyed com_number and its resources
// Receives:
//    portCHAR com_number: with the COMunication number port we want to close
// Returns:
// By reference:
// By value
void COM_vSerialClose( portCHAR com_number )
{

	switch (com_number){

		case COM_SERIAL_232:
			if (com_config.com_serial_232_initialized==TRUE){
				com_config.com_serial_232_initialized=FALSE;
				// release resources that we are not going to use
				vQueueDelete(x_serial_232_rx);
				vQueueDelete(x_serial_232_tx);

				// TODO SERIAL 232 USART interrupt should also be disabled when closing !!

			}//if
			break;

		case COM_SERIAL_USB:
			if (com_config.com_serial_USB_initialized==TRUE){
				com_config.com_serial_USB_initialized=FALSE;
				// release resources that we are not going to use
				vQueueDelete(x_serial_USB_rx);
				vQueueDelete(x_serial_USB_tx);

				// TODO SERIAL USB USART interrupt should also be disabled when closing !!

			}//if
			break;

		case COM_SERIAL_MIDI:
			if (com_config.com_serial_MIDI_initialized==TRUE){
				com_config.com_serial_MIDI_initialized=FALSE;
				// release resources that we are not going to use
				vQueueDelete(x_serial_MIDI_rx);
				vQueueDelete(x_serial_MIDI_tx);

				// TODO SERIAL MIDI USART interrupt should also be disabled when closing !!

			}//if
			break;

	}//switch

}//COM_vSerialClose



// Synth Core  20-11-2011  Tolaemon
// Function: void COM_flush(portCHAR com_number)
// Function wich removes all the chars stored into
// the com number specified buffer
// Receives:
// Returns:
void COM_flush(portCHAR com_number){
	portSHORT sh_aux;
	portCHAR  c_aux;


	switch (com_number){

		case COM_SERIAL_232:
			if (com_config.com_serial_232_initialized==TRUE){

				// remove all the elements stored into the queues ( the last condition is to
				// avoid blocking in case we are reciving bytes while we are removing them )
				sh_aux=0;
				if ( xQueueReceive( x_serial_232_rx, &c_aux, 0 ) && (sh_aux< COM_SERIAL_232_QUEUE_LENGTH) ){
					sh_aux++;
				}//if
				sh_aux=0;
				if ( xQueueReceive( x_serial_232_tx, &c_aux, 0 ) && (sh_aux< COM_SERIAL_232_QUEUE_LENGTH) ){
					sh_aux++;
				}//if

			}//if
			break;

		case COM_SERIAL_USB:
			if (com_config.com_serial_USB_initialized==TRUE){

				// remove all the elements stored into the queues ( the last condition is to
				// avoid blocking in case we are reciving bytes while we are removing them )
				sh_aux=0;
				if ( xQueueReceive( x_serial_USB_rx, &c_aux, 0 ) && (sh_aux< COM_SERIAL_USB_QUEUE_LENGTH) ){
					sh_aux++;
				}//if
				sh_aux=0;
				if ( xQueueReceive( x_serial_USB_tx, &c_aux, 0 ) && (sh_aux< COM_SERIAL_USB_QUEUE_LENGTH) ){
					sh_aux++;
				}//if

			}//if
			break;

		case COM_SERIAL_MIDI:
			if (com_config.com_serial_MIDI_initialized==TRUE){

				// remove all the elements stored into the queues ( the last condition is to
				// avoid blocking in case we are reciving bytes while we are removing them )
				sh_aux=0;
				if ( xQueueReceive( x_serial_MIDI_rx, &c_aux, 0 ) && (sh_aux< COM_SERIAL_MIDI_QUEUE_LENGTH) ){
					sh_aux++;
				}//if
				sh_aux=0;
				if ( xQueueReceive( x_serial_MIDI_tx, &c_aux, 0 ) && (sh_aux< COM_SERIAL_MIDI_QUEUE_LENGTH) ){
					sh_aux++;
				}//if

			}//if
			break;

	}//switch


}// COM_flush

