// *****************************************************************
// ****                      Synth CORE B                       ****
// ****                  www.tolaemon.com/scoreb                ****
// ****                       Source code                       ****
// ****                       13/11/2011                        ****
// ****                     Jordi Bartolome                     ****
// ****                                                         ****
// **** IMPORTANT:                                              ****
// **** Using parts of this code means accepting all conditions ****
// **** exposed on the web: www.tolaemon.com/scoreb             ****
// ****                                                         ****
// **** Parts of this code have been taken from the demo source ****
// **** code of EVK1100 Atmel's evaluation board				****
// *****************************************************************

/* Copyright (C) 2006-2008, Atmel Corporation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. The name of ATMEL may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ATMEL ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE EXPRESSLY AND
 * SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
	FreeRTOS.org V4.7.2 - Copyright (C) 2003-2008 Richard Barry.

	This file is part of the FreeRTOS.org distribution.

	FreeRTOS.org is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	FreeRTOS.org is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with FreeRTOS.org; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	A special exception to the GPL can be applied should you wish to distribute
	a combined work that includes FreeRTOS.org, without being obliged to provide
	the source code for any proprietary components.  See the licensing section
	of http://www.FreeRTOS.org for full details of how and when the exception
	can be applied.

	***************************************************************************

	Please ensure to read the configuration and relevant port sections of the
	online documentation.

	+++ http://www.FreeRTOS.org +++
	Documentation, latest information, license and contact details.

	+++ http://www.SafeRTOS.com +++
	A version that is certified for use in safety critical systems.

	+++ http://www.OpenRTOS.com +++
	Commercial support, development, porting, licensing and training services.

	***************************************************************************
*/

#ifndef SCOREB_CONSOLE_H
#define SCOREB_CONSOLE_H

#include "conf_explorer.h"

/*! \name Shell Commands
 */
//! @{
#define CMD_NONE              0x00
#define CMD_MOUNT             0x01
#define CMD_LS                0x02
#define CMD_CD                0x03
#define CMD_CAT               0x04
#define CMD_HELP              0x05
#define CMD_MKDIR             0x06
#define CMD_TOUCH             0x07
#define CMD_RM                0x08
#define CMD_APPEND            0x09
#define CMD_NB_DRIVE          0x0A
#define CMD_SET_ID            0x0B
#define CMD_GOTO_ID           0x0C
#define CMD_CP                0x0D
#define CMD_DF                0x0E
#define CMD_MV                0x0F
#define CMD_FORMAT            0x10
#define CMD_FAT               0x11
#define CMD_FORMAT32          0x12
#define CMD_TEST_SDRAM        0x13
#define CMD_GET_RTC_SECS      0x14
#define CMD_GET_RTC_TIME	  0x15
//! @}

/*! \name Special Char Values
 */
//! @{
#define CR                    '\r'
#define LF                    '\n'
#define CTRL_C                0x03
#define CTRL_Q                0x11
#define BKSPACE_CHAR          '\b'
#define ABORT_CHAR            CTRL_C
#define QUIT_APPEND           CTRL_Q
#define HISTORY_CHAR          '!'
//! @}

/*! \name String Values for Commands
 */
//! @{
#define STR_CD                "cd"
#define STR_MOUNT             "mount"
#define STR_CP                "cp"
#define STR_LS                "ls"
#define STR_RM                "rm"
#define STR_DF                "df"
#define STR_MKDIR             "mkdir"
#define STR_TOUCH             "touch"
#define STR_APPEND            "append"
#define STR_CAT               "cat"
#define STR_DISK              "disk"
#define STR_MARK              "mark"
#define STR_GOTO              "goto"
#define STR_MV                "mv"
#define STR_A                 "a:"
#define STR_B                 "b:"
#define STR_C                 "c:"
#define STR_D                 "d:"
#define STR_HELP              "help"
#define STR_FORMAT            "format"
#define STR_FORMAT32          "format32"
#define STR_FAT               "fat"
#define STR_TEST_SDRAM        "tsdram"
#define STR_GET_RTC_SEC       "rtcs"
#define STR_GET_RTC_TIME      "rtct"
//! @}

#define MSG_PROMPT            "$>"

#define MSG_CONSOLE_INFO	"\r\n\r\nSynth Core B Console\r\nBased on Atmel's AVR32 cmd routines\r\n"

#define MSG_WELCOME           "\r\n" \
                              "-------------------------\r\n" \
                              "    ATMEL AVR32 Shell\r\n" \
                              "-------------------------\r\n"
#define MSG_ER_CMD_NOT_FOUND  "Command not found\r\n"
#define MSG_ER_MOUNT          "Unable to mount drive\r\n"
#define MSG_ER_DRIVE          "Drive does not exist\r\n"
#define MSG_ER_RM             "Can not erase; if the name is a directory, check it is empty\r\n"
#define MSG_ER_UNKNOWN_FILE   "Unknown file\r\n"
#define MSG_ER_MV             "Error during move\r\n"
#define MSG_ER_FORMAT         "Format fails\r\n"
#define MSG_APPEND_WELCOME    "\r\nSimple text editor, enter char to append, ^q to exit and save\r\n"
#define MSG_HELP_TITLE		  "Synth Core B Help\r\n"
#define MSG_HELP              "Commands summary\r\n" \
                              " a:, b:... goto selected drive               mount disk(a, b...)\r\n" \
                              " cd dirname                                  ls\r\n" \
                              " mkdir dirname                               cat filename\r\n" \
                              " touch filename                              disk get number of drives\r\n" \
                              " append filename                             goto goto bookmark\r\n" \
                              " mark: bookmark current directory            df get free space information\r\n" \
                              " cp filename: copy filename to bookmark      fat get FAT type for current drive\r\n" \
                              " rm filename: erase file or EMPTY directory  format drivename, with drivename: a, b...\r\n" \
                              " mv src dst: move file or directory          format32 drivename, with drivename: a, b...\r\n" \
                              " tsdram checks sdram                         rtcs get RTC Unix time in secs. since 1970\r\n" \
							  " rtct get RTC time & date fields             help\r\n"

void CONSOLE_reset();
signed portCHAR CONSOLE_init(portCHAR device_type, portCHAR device_number);
void CONSOLE_close();
void CONSOLE_test_sdram();
void CONSOLE_append_file(void);
void CONSOLE_build_cmd(void);
void CONSOLE_parse_cmd(void);
static void CONSOLE_task( void *pvParameters );


#endif // SCOREB_CONSOLE_H

