// *****************************************************************
// ****                      Synth CORE B                       ****
// ****                  www.tolaemon.com/scoreb                ****
// ****                       Source code                       ****
// ****                       13/11/2011                        ****
// ****                     Jordi Bartolome                     ****
// ****                                                         ****
// **** IMPORTANT:                                              ****
// **** Using parts of this code means accepting all conditions ****
// **** exposed on the web: www.tolaemon.com/scoreb             ****
// ****                                                         ****
// **** Parts of this code have been taken from the demo source ****
// **** code of EVK1100 Atmel's evaluation board				****
// *****************************************************************

/* Copyright (C) 2006-2008, Atmel Corporation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. The name of ATMEL may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ATMEL ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE EXPRESSLY AND
 * SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
	FreeRTOS.org V4.7.2 - Copyright (C) 2003-2008 Richard Barry.

	This file is part of the FreeRTOS.org distribution.

	FreeRTOS.org is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	FreeRTOS.org is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with FreeRTOS.org; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	A special exception to the GPL can be applied should you wish to distribute
	a combined work that includes FreeRTOS.org, without being obliged to provide
	the source code for any proprietary components.  See the licensing section
	of http://www.FreeRTOS.org for full details of how and when the exception
	can be applied.

	***************************************************************************

	Please ensure to read the configuration and relevant port sections of the
	online documentation.

	+++ http://www.FreeRTOS.org +++
	Documentation, latest information, license and contact details.

	+++ http://www.SafeRTOS.com +++
	A version that is certified for use in safety critical systems.

	+++ http://www.OpenRTOS.com +++
	Commercial support, development, porting, licensing and training services.

	***************************************************************************
*/

#ifndef SCOREB_MIDI_H
#define SCOREB_MIDI_H

#define MIDI_QUEUE_LENGTH 16

// maximum length of a MIDI message ( be carefull with SysEx messages, whic can be longer than normal messages )
#define MIDI_MAX_MESSAGE 128

// type of MIDI messages
#define MIDI_VOICE_MESSAGE_TYPE 			1
#define MIDI_SYSTEM_COMMON_MESSAGE_TYPE 	2
#define MIDI_SYSTEM_REALTIME_MESSAGE_TYPE 	3

// MIDI message codes, and length of the messages
#define MIDI_COD_NOTE_OFF 		 0x80
#define MIDI_LEN_NOTE_OFF 		 3

#define MIDI_COD_NOTE_ON 		 0x90
#define MIDI_LEN_NOTE_ON 		 3

#define MIDI_COD_AFTER_TOUCH	 0xA0
#define MIDI_LEN_AFTER_TOUCH 	 3

#define MIDI_COD_CTRL_CHANGE	 0xB0
#define MIDI_LEN_CTRL_CHANGE	 3

#define MIDI_COD_PROGR_CHANGE	 0xC0
#define MIDI_LEN_PROGR_CHANGE	 2

#define MIDI_COD_CHAN_PRESURE 	 0xD0
#define MIDI_LEN_CHAN_PRESURE 	 2

#define MIDI_COD_PTICH_WHEEL	 0xE0
#define MIDI_LEN_PTICH_WHEEL	 3

#define MIDI_COD_SYSEX 			0xF0
#define MIDI_COD_END_SYSEX 		0xF7
#define MIDI_LEN_SYSEX			0xFFFF  // not defined length

// mask used to check if a byte is a status byte
#define MIDI_MASK_IS_A_STATUS_BYTE 	0x80
#define MIDI_MASK_VOICE_MESSAGE		0xF0
#define MIDI_MASK_CHANNEL			0x0F

// type defined to store the messages sent through MIDI events
typedef struct _t_MIDI_event{
	portCHAR type;
	portCHAR code;
	portCHAR channel;
	signed portSHORT value_1;
	signed portSHORT value_2;
}t_MIDI_event;

void MIDI_reset();
signed portCHAR MIDI_init(portCHAR device_type, portCHAR device_number);
signed portCHAR MIDI_dequeue_received_event(t_MIDI_event * MIDI_event);
signed portCHAR MIDI_enqueue_event_to_send(t_MIDI_event * pMIDI_event);
void MIDI_close();


#endif // SCOREB_MIDI

