// *****************************************************************
// ****                      Synth CORE B                       ****
// ****                  www.tolaemon.com/scoreb                ****
// ****                       Source code                       ****
// ****                       13/11/2011                        ****
// ****                     Jordi Bartolome                     ****
// ****                                                         ****
// **** IMPORTANT:                                              ****
// **** Using parts of this code means accepting all conditions ****
// **** exposed on the web: www.tolaemon.com/scoreb             ****
// ****                                                         ****
// **** Parts of this code have been taken from the demo source ****
// **** code of EVK1100 Atmel's evaluation board				****
// *****************************************************************

#include <avr32/io.h>
#include "board.h"
#include "gpio.h"
#include "rtc.h"
#include "time.h"

// scheduler includes.
#include "FreeRTOS.h"
#include "queue.h"
#include "task.h"

// synth core B includes
#include "defines.h"
#include "rtc_clock.h"


// RTCs configuration vars
struct {
	portCHAR 			rtc_initialized;
	unsigned portLONG	sec;// the time is calculated with the seconds stored into this var
	unsigned portSHORT  sec_since_last_read; // stores the number of seconds elapsed since the RTC was read by the application for last time
} rtc_info;



// Synth Core  04-12-2011  Tolaemon
// Function: signed portCHAR  RTC_CLOCK_set time(unsigned portSHORT year,unsigned portSHORT month, unsigned portSHORT day, unsigned portSHORT hour, unsigned portSHORT minute, unsigned portSHORT second){
// Function which receives the different parameters with the local time, converts them to seconds from 1970 ( Unix Time ) and stores them into rtc seconds counter
// Receives:
// Different parameters with the local time:
//  	unsigned portSHORT year : year-1900, so 79 means 1979, 103 means 2003
// 		unsigned portSHORT month : 0 to 11
// 		unsigned portSHORT day : 1 to 31
// 		unsigned portSHORT hour : 0 to 23
// 		unsigned portSHORT minute : 0 to 59
// 		unsigned portSHORT second  :  0 to 59 (or 60 for occasional rare leap-seconds)
// Returns:
// By value:
//  +1 the time was set into the RTC counter
//  -1 time could not be set into the RTC counter
signed portCHAR RTC_CLOCK_set_time(unsigned portSHORT hour, unsigned portSHORT minute, unsigned portSHORT second, unsigned portSHORT day, unsigned portSHORT month,unsigned portSHORT year){
	struct tm	 time_date_time;
	time_t time_in_seconds_since_1970;

	if ( (rtc_info.rtc_initialized==TRUE)&&(year>1970)&&(month>=1)&&(month<=12)&&(hour>=0)&&(hour<=23)&&(minute>=0)&&(minute<=59)&&(second>=0)&&(second<=59) ){

		time_date_time.tm_sec = second;// 0 to 59
		time_date_time.tm_min = minute;// 0 to 59
		time_date_time.tm_hour = hour;// 0 to 23
		time_date_time.tm_mday = day;//  1 to 31
		time_date_time.tm_mon = month-1;// 0 to 11, stupidly 0=January, 11=December
		time_date_time.tm_year = year-1900;// year-1900, so 79 means 1979, 103 means 2003

		//conver time_in_seconds_since_1970
		time_in_seconds_since_1970 = mktime(&time_date_time);

		rtc_info.sec=time_in_seconds_since_1970;

		return +1;

	}else{

		return -1;

	}//if

}//RTC_CLOCK_set_time



// Synth Core  04-12-2011  Tolaemon
// Function: signed portCHAR RTC_CLOCK_get_time(unsigned portSHORT * year,unsigned portSHORT * month, unsigned portSHORT * day, unsigned portSHORT * hour, unsigned portSHORT * minute, unsigned portSHORT * second){
// Function which returns by reference the current time fields. It returns the different elemets of the current time in "human format": year, month, day, months ...
// Receives:
// Returns:
// By value:
//  +1 if the time could be returned
//  -1 if the time could not be returned
// By reference:
// The different parameters with the local time:
//  	unsigned portSHORT year : year-1900, so 79 means 1979, 103 means 2003
// 		unsigned portSHORT month : 0 to 11
// 		unsigned portSHORT day : 1 to 31
// 		unsigned portSHORT hour : 0 to 23
// 		unsigned portSHORT minute : 0 to 59
// 		unsigned portSHORT second  :  0 to 59 (or 60 for occasional rare leap-seconds)
signed portCHAR RTC_CLOCK_get_time(unsigned portSHORT * hour, unsigned portSHORT * minute, unsigned portSHORT * second, unsigned portSHORT * day, unsigned portSHORT * month,unsigned portSHORT * year){
  time_t seconds_since_1970;
  struct tm time_date_time;

  if (rtc_info.rtc_initialized==FALSE){

	  return -1;

  }else{

	  seconds_since_1970=(time_t)rtc_info.sec;
	  time (&seconds_since_1970);
	  localtime_r(&seconds_since_1970, &time_date_time);

	  (*hour)=(unsigned portSHORT)time_date_time.tm_hour;// 0 to 23
	  (*minute)=(unsigned portSHORT)time_date_time.tm_min;// 0 to 59
	  (*second)=(unsigned portSHORT)time_date_time.tm_sec;// 0 to 59
	  (*day)=(unsigned portSHORT)time_date_time.tm_mday;//  1 to 31
	  (*month)=(unsigned portSHORT)(time_date_time.tm_mon+1);// 0 to 11, stupidly 0=January, 11=December
	  (*year)=(unsigned portSHORT)(1900+time_date_time.tm_year);// year-1900, so 79 means 1979, 103 means 2003

	  return 1;

  }//if

}//RTC_CLOCK_get_time



// Synth Core  28-11-2011  Tolaemon
// Function: void RTC_CLOCK_reset()
// Function wich resets the RTC_CLOCK control vars. It is used when
// intializing the system to grant all vars are on "initial conditions"
// Receives:
// Returns:
void RTC_CLOCK_reset(){

	rtc_info.rtc_initialized=FALSE;
	rtc_info.sec=0;
	rtc_info.sec_since_last_read=0;

}// RTC_CLOCK_reset



// Synth Core  28-11-2011  Tolaemon
// Function: void RTC_CLOCK_read()
// Function returns the value of the RTC seconds counter
// Receives:
// Returns:
// By value:
//   the number of seconds in the sec counter ( 0 if the
//   RTC timer is not initialized )
unsigned portLONG RTC_CLOCK_read(void){

	if (rtc_info.rtc_initialized==FALSE){
		return 0;
	}else{
		rtc_info.sec_since_last_read=0;
		return rtc_info.sec;
	}//if

}//RTC_CLOCK_read



//
// USART interrupt service routine.
//
/*
#if __GNUC__
	__attribute__((__naked__))
#elif __ICCAVR32__
	#pragma shadow_registers = full   // Naked.
#endif
*/
#if __GNUC__
__attribute__((__interrupt__))
#elif __ICCAVR32__
// RTC Interrupt
#pragma handler = AVR32_RTC_IRQ_GROUP, 1
__interrupt
#endif
static void RTC_CLOCK_irq(void)
{
  /* This ISR can cause a context switch, so the first statement must be a
  call to the portENTER_SWITCHING_ISR() macro.  This must be BEFORE any
  variable declarations.
  portENTER_SWITCHING_ISR();*/

  // Increment the sec counter
  rtc_info.sec++;
  rtc_info.sec_since_last_read++;

  // clear the interrupt flag
  rtc_clear_interrupt(&AVR32_RTC);

  /* Exit the ISR.  If a task was woken by either a character being received
  or transmitted then a context switch will occur.
  portEXIT_SWITCHING_ISR();*/

}//RTC_CLOCK_irq



// Synth Core  28-11-2011  Tolaemon
// Function: signed portCHAR RTC_CLOCK_init();
// Initializes and starts the Real Time Clock
// Receives:
// Returns:
// By value:
// 	 1: if the RTC was properly initialized
//  -1: if the RTC could not be initialized
// Note:
// This code corresponds to AVR32's RTC initialization
// sample code
signed portCHAR RTC_CLOCK_init(){

	// Disable all interrupts.
	Disable_global_interrupt();

	  // The INTC driver has to be used only for GNU GCC for AVR32.
	 #if __GNUC__

		  // Register the RTC interrupt handler to the interrupt controller.
		  INTC_register_interrupt(&RTC_CLOCK_irq, AVR32_RTC_IRQ, RTC_CLOCK_INTERRUPT_LEVEL);

	 #endif

	  // Initialize the RTC
	  if (!rtc_init(&AVR32_RTC, RTC_OSC_32KHZ, RTC_PSEL_32KHZ_1HZ))
	  {
	    // ERROR initializing the RTC
		return -1;
	  }

	  // Set top value to 0 to generate an interrupt every seconds
	  rtc_set_top_value(&AVR32_RTC, 0);

	  // Enable the interrupts
	  rtc_enable_interrupt(&AVR32_RTC);

	  // Enable the RTC
	  rtc_enable(&AVR32_RTC);

	  // Enable global interrupts
	  Enable_global_interrupt();

	  rtc_info.rtc_initialized=TRUE;

	  return 1;

}//RTC_CLOCK_init



// Synth Core  28-11-2011  Tolaemon
// Function: void RTC_CLOCK_close()
// Function which releases RTC resources and closes the RTC clock
// Receives:
// Returns:
void RTC_CLOCK_close(){

	rtc_info.rtc_initialized=FALSE;
	rtc_info.sec=0;
	rtc_info.sec_since_last_read=0;

	// TODO RTC interrupt should be disabled here !!

}//RTC_CLOCK_close
